"""Version information for LlamaAgent.

Author: Nik Jois <nikjois@llamasearch.ai>
"""

__version__ = "0.1.3"
__version_info__ = (0, 1, 3)

# Version components
MAJOR = 0
MINOR = 1
PATCH = 3

# Build metadata
BUILD_DATE = "2025-01-15"
BUILD_COMMIT = "current"
BUILD_BRANCH = "main"

# Compatibility
version = __version__
version_info = __version_info__
